% generate a mesh for the square (-1,1) x (-1,1)

[p,e,t]=initmesh('square1');
[p,e,t]=refinemesh('square1',p,e,t);
[p]=jigglemesh(p,e,t);

% generate mass and stiffness matrices

aaa = @(x,y) 1;

M = MassAssembler2D(p,t);
A = StiffnessAssembler2D(p,t,aaa);

% Note that these matrices incorporate nodes on the boundary

% The boundary points are found by means of the topology 
% matrix t generated by initmesh, ...

np = size(p,2); % total number of points / nodes
dirichlet = unique([e(1,:) e(2,:)]);        % boundary nodes
interior = setdiff([1:np],dirichlet);       % interior nodes

f = 4;
b = M*ones(np,1)*f; % This is more complicated for general f

% $$$ There is a pdetool alternative to assemble A and b
% $$$ [A,unused,b] = assema(p,t,1,0,f);   

u=zeros(np,1);     % memory for solution vector

% assign values at the Dirichlet boundary nodes
for i=dirichlet,   
   x = p(1,i); y = p(2,i);
   if abs(abs(x)-1) < 1e-6,
      u(i) = 1-y^2;                    % left/right boundary
   else
      u(i) = 1-x^2;                    % top/bottom boundary
   end
end
                                  % modify right-hand side b
b = b(interior) - A(interior,dirichlet)*u(dirichlet); 
A = A(interior,interior);                         % modify A
                                            
u(interior) = A\b;          % solve for interior node values

% plot the solution
pdeplot(p,e,t,'xydata',u,'zdata',u,'mesh','on',...
        'colormap','jet');
